@@ -20,22 +20,23 @@
  *
  */
 
-struct vvm_plugin_desc {
-	const char *name;
-	int (*init) (void);
-	void (*exit) (void);
+struct vvm_plugin_desc
+{
+  const char *name;
+  int (*init) (void);
+  void (*exit) (void);
 };
 
 #ifdef VVM_PLUGIN_BUILTIN
-#define VVM_PLUGIN_DEFINE(name, init, exit) \
-		struct vvm_plugin_desc __vvm_builtin_ ## name = { \
-			#name, init, exit \
-		};
+#define VVM_PLUGIN_DEFINE(name, init, exit)         \
+  struct vvm_plugin_desc __vvm_builtin_ ## name = { \
+    #name, init, exit                               \
+  };
 #else
-#define VVM_PLUGIN_DEFINE(name, init, exit) \
-		extern struct vvm_plugin_desc vvm_plugin_desc \
-				__attribute__ ((visibility("default"))); \
-		struct vvm_plugin_desc vvm_plugin_desc = { \
-			#name, init, exit \
-		};
+#define VVM_PLUGIN_DEFINE(name, init, exit)     \
+  extern struct vvm_plugin_desc vvm_plugin_desc \
+  __attribute__ ((visibility ("default")));     \
+  struct vvm_plugin_desc vvm_plugin_desc = {    \
+    #name, init, exit                           \
+  };
 #endif
