@@ -54,6 +54,7 @@
 #include "dbus.h"
 #include "vvmutil.h"
 #include "phone-utils.h"
+#include "resolve.h"
 
 #define BEARER_SETUP_TIMEOUT    20      /* 20 seconds */
 #define BEARER_IDLE_TIMEOUT     10      /* 10 seconds */
@@ -67,7 +68,6 @@ struct vvm_service
 {
   char *identity;
   char *path;
-  char *interface;
   char *country_code;
   char *own_number;
   char *mailbox_interface;
@@ -83,7 +83,8 @@ struct vvm_service
   char *mailbox_TUI_password_length;
   char *mailbox_vvm_type;
   char *mailbox_ca_bundle;
-  int mailbox_starttls;
+  char *resolvers_ipv4_csv;
+  char *resolvers_ipv6_csv;
   int mailbox_legacytls;
   vvm_service_bearer_handler_func_t bearer_handler;
   void *bearer_data;
@@ -92,10 +93,10 @@ struct vvm_service
   int allow_plaintext_connection;
   gboolean bearer_setup;
   gboolean bearer_active;
-  gboolean use_mailbox_interface;
   GHashTable *messages;
   GKeyFile *settings;
   CURL *curl;
+  struct curl_slist *host_slist;
 };
 
 static GList *service_list;
@@ -465,26 +466,124 @@ vvm_service_set_subscription_configuration (struct vvm_service *service,
                          "MailboxVVMType", service->mailbox_vvm_type);
 }
 
+int
+vvm_service_set_resolvers (struct vvm_service *service,
+                           const gchar        *ipv4_csv,
+                           const gchar        *ipv6_csv)
+{
+  DBG ("service %p resolvers: ipv4: %s, ipv6: %s", service, ipv4_csv, ipv6_csv);
+
+  if (service == NULL)
+    return -EINVAL;
+
+  g_clear_pointer (&service->resolvers_ipv4_csv, g_free);
+  if (ipv4_csv && *ipv4_csv)
+    service->resolvers_ipv4_csv = g_strdup (ipv4_csv);
+
+  g_clear_pointer (&service->resolvers_ipv6_csv, g_free);
+  if (ipv6_csv && *ipv6_csv)
+    service->resolvers_ipv6_csv = g_strdup (ipv6_csv);
+
+  return 0;
+}
+
+static void
+vvm_service_set_host_resolve (struct vvm_service *service)
+{
+  g_autofree char *ip = NULL;
+  g_autofree char *host_list = NULL;
+  int ip_type;
+
+  curl_slist_free_all (service->host_slist);
+  service->host_slist = NULL;
+
+  ip = vvm_resolve_resolve_host (service->mailbox_hostname,
+                                 service->mailbox_interface,
+                                 service->resolvers_ipv4_csv,
+                                 service->resolvers_ipv6_csv);
+
+  DBG ("Host %s IP: %s", service->mailbox_hostname, ip);
+
+  if (!ip)
+    return;
+
+  ip_type = vvm_resolve_ip_version (ip);
+
+  if (ip_type == 6)
+    {
+      g_autofree char *new_ip = NULL;
+
+      new_ip = g_strdup_printf ("[%s]", ip);
+      g_free (ip);
+      ip = g_strdup (new_ip);
+    }
+  else if (ip_type == 4)
+     DBG ("IP is v4");
+  else {
+    g_warning ("Cannot figure out if IP `%s` is ipv4 or ipv6", ip);
+    return;
+  }
+
+  host_list = g_strdup_printf ("+%s:993:%s", service->mailbox_hostname, ip);
+  service->host_slist = curl_slist_append(service->host_slist, host_list);
+  g_clear_pointer (&host_list, g_free);
+  host_list = g_strdup_printf ("+%s:143:%s", service->mailbox_hostname, ip);
+  service->host_slist = curl_slist_append(service->host_slist, host_list);
+  g_clear_pointer (&host_list, g_free);
+  if (service->mailbox_port)
+    {
+      g_autoptr(GError) error = NULL;
+      guint64 port;
+      g_ascii_string_to_unsigned (service->mailbox_port, 10, 1, 65535, &port, &error);
+
+      if (error)
+        g_warning ("Error converting string `%s` to unsigned 64: %s", service->mailbox_port, error->message);
+      else if (port != 993 && port != 143)
+        {
+          host_list = g_strdup_printf ("+%s:%s:%s", service->mailbox_hostname, service->mailbox_port, ip);
+          service->host_slist = curl_slist_append(service->host_slist, host_list);
+          g_clear_pointer (&host_list, g_free);
+        }
+    }
+  curl_easy_setopt(service->curl, CURLOPT_RESOLVE, service->host_slist);
+}
+
+
+static CURL
+*create_curl_structure (struct vvm_service *service)
+{
+  CURL *curl;
+  curl = curl_easy_init ();
+  if (global_debug)
+    curl_easy_setopt (curl, CURLOPT_VERBOSE, 1L);
+
+  // Set Username and Password
+  curl_easy_setopt (curl, CURLOPT_USERNAME, service->mailbox_username);
+  curl_easy_setopt (curl, CURLOPT_PASSWORD, service->mailbox_password);
+  //Do five second timeout
+  curl_easy_setopt (curl, CURLOPT_TIMEOUT, 5L);
+
+  curl_easy_setopt (curl, CURLOPT_INTERFACE, service->mailbox_interface);
+  if ((service->mailbox_ca_bundle != NULL) && (service->mailbox_ca_bundle[0] != '\0'))
+    curl_easy_setopt (curl, CURLOPT_CAINFO, service->mailbox_ca_bundle);
+
+  return curl;
+}
+
 static void
 activate_curl_structure (struct vvm_service *service)
 {
   DBG ("Setting up CURL...");
-  service->curl = curl_easy_init ();
-  if (global_debug)
-    curl_easy_setopt (service->curl, CURLOPT_VERBOSE, 1L);
-  curl_easy_setopt (service->curl, CURLOPT_USERNAME, service->mailbox_username);
-  curl_easy_setopt (service->curl, CURLOPT_PASSWORD, service->mailbox_password);
-  if (service->mailbox_starttls)
+  service->curl = create_curl_structure (service);
+
+  if (!service->allow_plaintext_connection)
     curl_easy_setopt (service->curl, CURLOPT_USE_SSL, CURLUSESSL_ALL);
+  else
+      g_warning ("Allowing plaintext connection over IMAP");
   if (service->mailbox_legacytls)
     curl_easy_setopt (service->curl, CURLOPT_SSL_CIPHER_LIST, "ALL:@SECLEVEL=1");
-  if (service->use_mailbox_interface)
-    curl_easy_setopt (service->curl, CURLOPT_INTERFACE, service->mailbox_interface);
-
   if (*service->mailbox_auth)
     curl_easy_setopt (service->curl, CURLOPT_LOGIN_OPTIONS, service->mailbox_auth);
-  if ((service->mailbox_ca_bundle != NULL) && (service->mailbox_ca_bundle[0] != '\0'))
-    curl_easy_setopt (service->curl, CURLOPT_CAINFO, service->mailbox_ca_bundle);
 }
 
 static void
@@ -511,12 +610,72 @@ vvm_service_deactivate_vvm_imap_server (struct vvm_service *service)
   vvm_settings_sync (service->identity, SETTINGS_STORE, service->settings);
 }
 
+/*
+ * The IMAP setting succeeds, URI and mailbox_auth are transferred to a
+ * different pointer. As such, only free them if this function returns false.
+ * curl_easy_cleanup () is also performed if this returns TRUE.
+ */
+static gboolean
+vvm_service_try_imap_setting (struct vvm_service *service,
+                              CURL *curl,
+                              char *URI,
+                              char *mailbox_auth)
+{
+  CURLcode res = CURLE_OK;
+
+  curl_easy_setopt (curl, CURLOPT_URL, URI);
+  if (mailbox_auth && *mailbox_auth)
+    curl_easy_setopt (curl, CURLOPT_LOGIN_OPTIONS, mailbox_auth);
+  else
+    curl_easy_setopt (curl, CURLOPT_LOGIN_OPTIONS, NULL);
+
+  vvm_service_set_host_resolve (service);
+
+  res = curl_easy_perform (curl);
+  // Try to guess if the server is using legacy ciphers
+  if (res == CURLE_SSL_CONNECT_ERROR)
+    {
+      DBG ("curl_easy_perform() failed with SSL Error");
+      DBG ("Trying with legacy ciphers");
+      curl_easy_setopt (curl, CURLOPT_SSL_CIPHER_LIST, "ALL:@SECLEVEL=1");
+      res = curl_easy_perform (curl);
+      if (res == CURLE_OK)
+        {
+          service->mailbox_legacytls = TRUE;
+          g_key_file_set_boolean (service->settings, SETTINGS_GROUP_SERVICE,
+                                  "MailboxLegacyTLS", service->mailbox_legacytls);
+        }
+    }
+
+  if (res == CURLE_OK)
+    {
+      DBG ("Mailbox Activated!");
+      service->mailbox_active = TRUE;
+      service->mailbox_URI = URI;
+      service->mailbox_auth = mailbox_auth;
+
+      g_key_file_set_boolean (service->settings, SETTINGS_GROUP_SERVICE,
+                              "MailboxActive", service->mailbox_active);
+      g_key_file_set_string (service->settings, SETTINGS_GROUP_SERVICE,
+                             "MailboxURI", service->mailbox_URI);
+      g_key_file_set_string (service->settings, SETTINGS_GROUP_SERVICE,
+                             "MailboxAuth", service->mailbox_auth);
+      curl_easy_cleanup (curl);
+      vvm_settings_sync (service->identity, SETTINGS_STORE, service->settings);
+      return TRUE;
+    }
+
+  /* Reset authorized ciphers to the default value */
+  curl_easy_setopt (curl, CURLOPT_SSL_CIPHER_LIST, NULL);
+  g_warning ("curl_easy_perform() failed: %s\n", curl_easy_strerror (res));
+
+  return FALSE;
+}
+
 static int
-vvm_service_test_vvm_imap_server (struct vvm_service *service,
-                                  gboolean            modem_iface)
+vvm_service_activate_vvm_imap_server (struct vvm_service *service)
 {
   CURL *curl;
-  CURLcode res = CURLE_OK;
   char *URI, *mailbox_auth;
 
   DBG ("Activating IMAP Mailbox");
@@ -526,352 +685,95 @@ vvm_service_test_vvm_imap_server (struct vvm_service *service,
       return TRUE;
     }
 
-  curl = curl_easy_init ();
-  if (global_debug)
-    curl_easy_setopt (curl, CURLOPT_VERBOSE, 1L);
-
-  // Set Username and Password
-  curl_easy_setopt (curl, CURLOPT_USERNAME, service->mailbox_username);
-  curl_easy_setopt (curl, CURLOPT_PASSWORD, service->mailbox_password);
-  //Do five second timeout
-  curl_easy_setopt (curl, CURLOPT_TIMEOUT, 5L);
-
-  if (modem_iface)
-    curl_easy_setopt (curl, CURLOPT_INTERFACE, service->mailbox_interface);
-  if ((service->mailbox_ca_bundle != NULL) && (service->mailbox_ca_bundle[0] != '\0'))
-    curl_easy_setopt (curl, CURLOPT_CAINFO, service->mailbox_ca_bundle);
-
+  curl = create_curl_structure (service);
   /*
    * This is testing various IMAP settings to see that works in this order:
    * IMAP over SSL:
    * curl -v imaps://$USERNAME:$PASSWORD@$HOSTNAME/
+   * IMAP with non-SASL LOGIN
+   * curl -v --login-options AUTH=+LOGIN imaps://$USERNAME:$PASSWORD@$HOSTNAME/
    * IMAP over SSL (AUTH=PLAIN):
    * curl -v --login-options AUTH=PLAIN imaps://$USERNAME:$PASSWORD@$HOSTNAME/
    * IMAP over SSL (AUTH=DIGEST-MD5):
    * curl -v --login-options AUTH=DIGEST-MD5 imaps://$USERNAME:$PASSWORD@$HOSTNAME/
-   * IMAP on the port provided by the STATUS SMS:
-   * curl -v imap://$USERNAME:$PASSWORD@$HOSTNAME:$PORT/
-   * IMAP over the standard port (If the provided port is different)
+   * IMAP over the standard port:
    * curl -v imap://$USERNAME:$PASSWORD@$HOSTNAME/
+   * IMAP on the port provided by the STATUS SMS  (If the provided port is different):
+   * curl -v imap://$USERNAME:$PASSWORD@$HOSTNAME:$PORT/
    *
+   * Note that on Android AOSP, non-SASL LOGIN and DIGEST-MD5 are the only ones
+   * supported
    */
 
-  //curl -v imaps://$USERNAME:$PASSWORD@$HOSTNAME/
+  /* curl -v imaps://$USERNAME:$PASSWORD@$HOSTNAME/ */
   DBG ("Trying IMAP with SSL");
 
   mailbox_auth = g_strdup ("");
   URI = g_strdup_printf ("imaps://%s/INBOX", service->mailbox_hostname);
-  curl_easy_setopt (curl, CURLOPT_URL, URI);
 
-  res = curl_easy_perform (curl);
-  // Try to guess if the server is using legacy ciphers
-  if (res == CURLE_SSL_CONNECT_ERROR)
-    {
-      DBG ("curl_easy_perform() failed: %s\n", curl_easy_strerror (res));
-      DBG ("Trying with legacy ciphers");
-      curl_easy_setopt (curl, CURLOPT_SSL_CIPHER_LIST, "ALL:@SECLEVEL=1");
-      res = curl_easy_perform (curl);
-      if (res == CURLE_OK)
-        {
-          service->mailbox_legacytls = TRUE;
-          g_key_file_set_boolean (service->settings, SETTINGS_GROUP_SERVICE,
-                                  "MailboxLegacyTLS", service->mailbox_legacytls);
-        }
-    }
-  if (res == CURLE_OK)
-    {
-      DBG ("Mailbox Activated!");
-      service->mailbox_active = TRUE;
-      service->mailbox_URI = URI;
-      service->mailbox_auth = mailbox_auth;
+  if (vvm_service_try_imap_setting (service, curl, URI, mailbox_auth))
+    return TRUE;
 
-      g_key_file_set_boolean (service->settings, SETTINGS_GROUP_SERVICE,
-                              "MailboxActive", service->mailbox_active);
-      g_key_file_set_string (service->settings, SETTINGS_GROUP_SERVICE,
-                             "MailboxURI", service->mailbox_URI);
-      g_key_file_set_string (service->settings, SETTINGS_GROUP_SERVICE,
-                             "MailboxAuth", service->mailbox_auth);
-      curl_easy_cleanup (curl);
-      return TRUE;
-    }
-  else {
-      // Reset authorized ciphers to the default value
-      curl_easy_setopt (curl, CURLOPT_SSL_CIPHER_LIST, "ALL");
-      g_warning ("curl_easy_perform() failed: %s\n", curl_easy_strerror (res));
-    }
-
-  //curl -v --login-options AUTH=+LOGIN imaps://$USERNAME:$PASSWORD@$HOSTNAME/
+  /* curl -v --login-options AUTH=+LOGIN imaps://$USERNAME:$PASSWORD@$HOSTNAME/ */
   DBG ("Trying IMAP with SSL: Explicit +LOGIN Authentication");
   g_free (mailbox_auth);
   mailbox_auth = g_strdup ("AUTH=+LOGIN");
-  curl_easy_setopt (curl, CURLOPT_LOGIN_OPTIONS, mailbox_auth);
 
-  res = curl_easy_perform (curl);
-  // Try to guess if the server is using legacy ciphers
-  if (res == CURLE_SSL_CONNECT_ERROR)
-    {
-      DBG ("curl_easy_perform() failed: %s\n", curl_easy_strerror (res));
-      DBG ("Trying with legacy ciphers");
-      curl_easy_setopt (curl, CURLOPT_SSL_CIPHER_LIST, "ALL:@SECLEVEL=1");
-      res = curl_easy_perform (curl);
-      if (res == CURLE_OK)
-        {
-          service->mailbox_legacytls = TRUE;
-          g_key_file_set_boolean (service->settings, SETTINGS_GROUP_SERVICE,
-                                  "MailboxLegacyTLS", service->mailbox_legacytls);
-        }
-    }
-  if (res == CURLE_OK)
-    {
-      DBG ("Mailbox Activated!");
-      service->mailbox_active = TRUE;
-      service->mailbox_URI = URI;
-      service->mailbox_auth = mailbox_auth;
+  if (vvm_service_try_imap_setting (service, curl, URI, mailbox_auth))
+    return TRUE;
 
-      g_key_file_set_boolean (service->settings, SETTINGS_GROUP_SERVICE,
-                              "MailboxActive", service->mailbox_active);
-      g_key_file_set_string (service->settings, SETTINGS_GROUP_SERVICE,
-                             "MailboxURI", service->mailbox_URI);
-      g_key_file_set_string (service->settings, SETTINGS_GROUP_SERVICE,
-                             "MailboxAuth", service->mailbox_auth);
-      curl_easy_cleanup (curl);
-      return TRUE;
-    }
-  else {
-      // Reset authorized ciphers to the default value
-      curl_easy_setopt (curl, CURLOPT_SSL_CIPHER_LIST, "ALL");
-      g_warning ("curl_easy_perform() failed: %s\n", curl_easy_strerror (res));
-    }
-
-  //curl -v --login-options AUTH=PLAIN imaps://$USERNAME:$PASSWORD@$HOSTNAME/
+  /* curl -v --login-options AUTH=PLAIN imaps://$USERNAME:$PASSWORD@$HOSTNAME/ */
   DBG ("Trying IMAP with SSL: Explicit PLAIN Authentication");
   g_free (mailbox_auth);
   mailbox_auth = g_strdup ("AUTH=PLAIN");
-  curl_easy_setopt (curl, CURLOPT_LOGIN_OPTIONS, mailbox_auth);
 
-  res = curl_easy_perform (curl);
-  // Try to guess if the server is using legacy ciphers
-  if (res == CURLE_SSL_CONNECT_ERROR)
-    {
-      DBG ("curl_easy_perform() failed: %s\n", curl_easy_strerror (res));
-      DBG ("Trying with legacy ciphers");
-      curl_easy_setopt (curl, CURLOPT_SSL_CIPHER_LIST, "ALL:@SECLEVEL=1");
-      res = curl_easy_perform (curl);
-      if (res == CURLE_OK)
-        {
-          service->mailbox_legacytls = TRUE;
-          g_key_file_set_boolean (service->settings, SETTINGS_GROUP_SERVICE,
-                                  "MailboxLegacyTLS", service->mailbox_legacytls);
-        }
-    }
-  if (res == CURLE_OK)
-    {
-      DBG ("Mailbox Activated!");
-      service->mailbox_active = TRUE;
-      service->mailbox_URI = URI;
-      service->mailbox_auth = mailbox_auth;
+  if (vvm_service_try_imap_setting (service, curl, URI, mailbox_auth))
+    return TRUE;
 
-      g_key_file_set_boolean (service->settings, SETTINGS_GROUP_SERVICE,
-                              "MailboxActive", service->mailbox_active);
-      g_key_file_set_string (service->settings, SETTINGS_GROUP_SERVICE,
-                             "MailboxURI", service->mailbox_URI);
-      g_key_file_set_string (service->settings, SETTINGS_GROUP_SERVICE,
-                             "MailboxAuth", service->mailbox_auth);
-      curl_easy_cleanup (curl);
-      return TRUE;
-    }
-  else {
-      // Reset authorized ciphers to the default value
-      curl_easy_setopt (curl, CURLOPT_SSL_CIPHER_LIST, "ALL");
-      g_warning ("curl_easy_perform() failed: %s\n", curl_easy_strerror (res));
-    }
-
-  //curl -v --login-options AUTH=DIGEST-MD5 imaps://$USERNAME:$PASSWORD@$HOSTNAME/
+  /* curl -v --login-options AUTH=DIGEST-MD5 imaps://$USERNAME:$PASSWORD@$HOSTNAME/ */
   DBG ("Trying IMAP with SSL: Explicit DIGEST-MD5 Authentication");
   g_free (mailbox_auth);
   mailbox_auth = g_strdup ("AUTH=DIGEST-MD5");
-  curl_easy_setopt (curl, CURLOPT_LOGIN_OPTIONS, mailbox_auth);
-  res = curl_easy_perform (curl);
-  // Try to guess if the server is using legacy ciphers
-  if (res == CURLE_SSL_CONNECT_ERROR)
-    {
-      DBG ("curl_easy_perform() failed: %s\n", curl_easy_strerror (res));
-      DBG ("Trying with legacy ciphers");
-      curl_easy_setopt (curl, CURLOPT_SSL_CIPHER_LIST, "ALL:@SECLEVEL=1");
-      res = curl_easy_perform (curl);
-      if (res == CURLE_OK)
-        {
-          service->mailbox_legacytls = TRUE;
-          g_key_file_set_boolean (service->settings, SETTINGS_GROUP_SERVICE,
-                                  "MailboxLegacyTLS", service->mailbox_legacytls);
-        }
-    }
-  if (res == CURLE_OK)
-    {
-      DBG ("Mailbox Activated!");
-      service->mailbox_active = TRUE;
-      service->mailbox_URI = URI;
-      service->mailbox_auth = mailbox_auth;
 
-      g_key_file_set_boolean (service->settings, SETTINGS_GROUP_SERVICE,
-                              "MailboxActive", service->mailbox_active);
-      g_key_file_set_string (service->settings, SETTINGS_GROUP_SERVICE,
-                             "MailboxURI", service->mailbox_URI);
-      g_key_file_set_string (service->settings, SETTINGS_GROUP_SERVICE,
-                             "MailboxAuth", service->mailbox_auth);
-      curl_easy_cleanup (curl);
-      return TRUE;
-    }
-  else {
-      // Reset authorized ciphers to the default value
-      curl_easy_setopt (curl, CURLOPT_SSL_CIPHER_LIST, "ALL");
-      g_warning ("curl_easy_perform() failed: %s\n", curl_easy_strerror (res));
-    }
+  if (vvm_service_try_imap_setting (service, curl, URI, mailbox_auth))
+    return TRUE;
 
-  // I need to clean out the curl_easy_setopt(), so tear down and re-setup
-  curl_easy_cleanup (curl);
-  curl = curl_easy_init ();
-  if (global_debug)
-    curl_easy_setopt (curl, CURLOPT_VERBOSE, 1L);
-
-  // Set Username and Password
-  curl_easy_setopt (curl, CURLOPT_USERNAME, service->mailbox_username);
-  curl_easy_setopt (curl, CURLOPT_PASSWORD, service->mailbox_password);
-  if ((service->mailbox_ca_bundle != NULL) && (service->mailbox_ca_bundle[0] != '\0'))
-    curl_easy_setopt (curl, CURLOPT_CAINFO, service->mailbox_ca_bundle);
-  //Do five second timeout
-  curl_easy_setopt (curl, CURLOPT_TIMEOUT, 5L);
-
-  if (modem_iface)
-    curl_easy_setopt (curl, CURLOPT_INTERFACE, service->mailbox_interface);
+  /*
+   * We are trying imap now instad of imaps, so I have to reset a lot of the curl stuff.
+   * I need to clean out the curl_easy_setopt(), so tear down and re-setup
+   */
 
   /* Do not allow any non-TLS connection */
   if (!service->allow_plaintext_connection)
-    {
-      DBG ("Disallowing plaintext connection over IMAP");
-      curl_easy_setopt (curl, CURLOPT_USE_SSL, CURLUSESSL_ALL);
-    }
+    curl_easy_setopt (curl, CURLOPT_USE_SSL, CURLUSESSL_ALL);
   else
-    DBG ("Allowing IMAP to choose if it wants a plaintext connection");
+    g_warning ("Allowing plaintext connection over IMAP");
 
   g_free (mailbox_auth);
   mailbox_auth = g_strdup ("");
 
-  DBG ("Trying IMAP with Specified port");
-  //curl -v imap://$USERNAME:$PASSWORD@$HOSTNAME:$PORT/
+  DBG ("Trying IMAP on default port");
+  /* curl -v imap://$USERNAME:$PASSWORD@$HOSTNAME:$PORT */
   g_free (URI);
-  URI = g_strdup_printf ("imap://%s:%s/INBOX", service->mailbox_hostname, service->mailbox_port);
-  curl_easy_setopt (curl, CURLOPT_URL, URI);
-  res = curl_easy_perform (curl);
-  // Try to guess if the server is using legacy ciphers
-  if (res == CURLE_SSL_CONNECT_ERROR)
-    {
-      DBG ("curl_easy_perform() failed: %s\n", curl_easy_strerror (res));
-      DBG ("Trying with legacy ciphers");
-      curl_easy_setopt (curl, CURLOPT_SSL_CIPHER_LIST, "ALL:@SECLEVEL=1");
-      res = curl_easy_perform (curl);
-      if (res == CURLE_OK)
-        {
-          service->mailbox_legacytls = TRUE;
-          g_key_file_set_boolean (service->settings, SETTINGS_GROUP_SERVICE,
-                                  "MailboxLegacyTLS", service->mailbox_legacytls);
-        }
-    }
-  if (res == CURLE_OK)
-    {
-      DBG ("Mailbox Activated!");
-      service->mailbox_active = TRUE;
-      service->mailbox_URI = URI;
-      service->mailbox_auth = mailbox_auth;
-      g_key_file_set_boolean (service->settings, SETTINGS_GROUP_SERVICE,
-                              "MailboxActive", service->mailbox_active);
-      g_key_file_set_string (service->settings, SETTINGS_GROUP_SERVICE,
-                             "MailboxURI", service->mailbox_URI);
-      g_key_file_set_string (service->settings, SETTINGS_GROUP_SERVICE,
-                             "MailboxAuth", service->mailbox_auth);
-      curl_easy_cleanup (curl);
-      return TRUE;
-    }
-  else {
-      // Reset authorized ciphers to the default value
-      curl_easy_setopt (curl, CURLOPT_SSL_CIPHER_LIST, "ALL");
-      g_warning ("curl_easy_perform() failed: %s\n", curl_easy_strerror (res));
-    }
+  URI = g_strdup_printf ("imap://%s/INBOX", service->mailbox_hostname);
+
+  if (vvm_service_try_imap_setting (service, curl, URI, mailbox_auth))
+    return TRUE;
 
   //Next, IMAP with default port
   if (g_strcmp0 (service->mailbox_port, "143") != 0)        //service->mailbox_port isn't 143 (default IMAP port)
     {
-      DBG ("Trying IMAP on default port");
-      //curl -v imap://$USERNAME:$PASSWORD@$HOSTNAME/
+      DBG ("Trying IMAP on specified port");
+      /* curl -v imap://$USERNAME:$PASSWORD@$HOSTNAME/ */
       g_free (URI);
-      URI = g_strdup_printf ("imap://%s/INBOX", service->mailbox_hostname);
-      curl_easy_setopt (curl, CURLOPT_URL, URI);
-      res = curl_easy_perform (curl);
-      // Try to guess if the server is using legacy ciphers
-      if (res == CURLE_SSL_CONNECT_ERROR)
-        {
-          DBG ("curl_easy_perform() failed: %s\n", curl_easy_strerror (res));
-          DBG ("Trying with legacy ciphers");
-          curl_easy_setopt (curl, CURLOPT_SSL_CIPHER_LIST, "ALL:@SECLEVEL=1");
-          res = curl_easy_perform (curl);
-          if (res == CURLE_OK)
-            {
-              service->mailbox_legacytls = TRUE;
-              g_key_file_set_boolean (service->settings, SETTINGS_GROUP_SERVICE,
-                                      "MailboxLegacyTLS", service->mailbox_legacytls);
-            }
-        }
-      if (res == CURLE_OK)
-        {
-          DBG ("Mailbox Activated!");
-          service->mailbox_active = TRUE;
-          service->mailbox_URI = URI;
-          service->mailbox_auth = mailbox_auth;
-          g_key_file_set_boolean (service->settings, SETTINGS_GROUP_SERVICE,
-                                  "MailboxActive", service->mailbox_active);
-          g_key_file_set_string (service->settings, SETTINGS_GROUP_SERVICE,
-                                 "MailboxURI", service->mailbox_URI);
-          g_key_file_set_string (service->settings, SETTINGS_GROUP_SERVICE,
-                                 "MailboxAuth", service->mailbox_auth);
-          curl_easy_cleanup (curl);
-          return TRUE;
-        }
-      else {
-          // Reset authorized ciphers to the default value
-          curl_easy_setopt (curl, CURLOPT_SSL_CIPHER_LIST, "ALL");
-          g_warning ("curl_easy_perform() failed: %s\n", curl_easy_strerror (res));
-        }
+      URI = g_strdup_printf ("imap://%s:%s/INBOX", service->mailbox_hostname, service->mailbox_port);
+      if (vvm_service_try_imap_setting (service, curl, URI, mailbox_auth))
+        return TRUE;
     }
 
   g_free (URI);
   g_free (mailbox_auth);
   curl_easy_cleanup (curl);
-  return FALSE;
-}
-
-static int
-vvm_service_activate_vvm_imap_server (struct vvm_service *service)
-{
-  service->use_mailbox_interface = FALSE;
-  DBG ("Trying to activate without using mailbox interface.");
-  if (vvm_service_test_vvm_imap_server (service, service->use_mailbox_interface))
-    {
-      g_key_file_set_boolean (service->settings, SETTINGS_GROUP_SERVICE,
-                              "UseMailboxInterface", service->use_mailbox_interface);
-      vvm_settings_sync (service->identity, SETTINGS_STORE, service->settings);
-      return TRUE;
-    }
-  DBG ("Retrying to activate without using mailbox interface.");
-  service->use_mailbox_interface = TRUE;
-  if (vvm_service_test_vvm_imap_server (service, service->use_mailbox_interface))
-    {
-      g_key_file_set_boolean (service->settings, SETTINGS_GROUP_SERVICE,
-                              "UseMailboxInterface", service->use_mailbox_interface);
-      vvm_settings_sync (service->identity, SETTINGS_STORE, service->settings);
-      return TRUE;
-    }
-  g_critical ("Unable to activate mailbox!");
-  g_critical ("You may not get any SMS SYNC Messages until the mailbox is active.");
   vvm_settings_sync (service->identity, SETTINGS_STORE, service->settings);
   return FALSE;
 }
@@ -966,6 +868,8 @@ vvmd_service_retrieve_headers (struct vvm_service         *service,
   curl_easy_setopt (service->curl, CURLOPT_URL, URI);
   curl_easy_setopt (service->curl, CURLOPT_WRITEFUNCTION, curl_string_cb);
   curl_easy_setopt (service->curl, CURLOPT_WRITEDATA, (void *)&chunk);
+  vvm_service_set_host_resolve (service);
+
   res = curl_easy_perform (service->curl);
   if (res == CURLE_OK)
     {
@@ -1092,7 +996,9 @@ vvm_service_retrieve_vvm_email (struct vvm_service *service,
   curl_easy_setopt (service->curl, CURLOPT_URL, URI);
   curl_easy_setopt (service->curl, CURLOPT_WRITEFUNCTION, curl_string_cb);
   curl_easy_setopt (service->curl, CURLOPT_WRITEDATA, (void *)&chunk);
+  vvm_service_set_host_resolve (service);
   res = curl_easy_perform (service->curl);
+
   if (res == CURLE_OK)
     {
       g_critical ("Downloaded email Successfully");
@@ -1309,8 +1215,9 @@ vvm_service_delete_vvm_email_from_server (struct vvm_service *service,
   curl_easy_setopt (service->curl, CURLOPT_URL, URI);
   DBG ("Delete Command: %s", delete_command);
   curl_easy_setopt (service->curl, CURLOPT_CUSTOMREQUEST, delete_command);
-
+  vvm_service_set_host_resolve (service);
   res = curl_easy_perform (service->curl);
+
   if (res == CURLE_OK)
     {
       DBG ("Deleted email Successfully, now expunging everything");
@@ -1512,8 +1419,9 @@ vvm_service_sync_vvm_imap_server (struct vvm_service *service)
   curl_easy_setopt (service->curl, CURLOPT_URL, URI);
   curl_easy_setopt (service->curl, CURLOPT_WRITEFUNCTION, curl_string_cb);
   curl_easy_setopt (service->curl, CURLOPT_WRITEDATA, (void *)&chunk);
-
+  vvm_service_set_host_resolve (service);
   res = curl_easy_perform (service->curl);
+
   if (res == CURLE_OK)
     {
       g_auto(GStrv) tokens = NULL;
@@ -1665,30 +1573,6 @@ vvm_load_settings (struct vvm_service *service)
       g_clear_error (&error);
     }
 
-  service->use_mailbox_interface = g_key_file_get_boolean (service->settings,
-                                                           SETTINGS_GROUP_SERVICE,
-                                                           "UseMailboxInterface", &error);
-  if (error)
-    {
-      service->use_mailbox_interface = FALSE;
-
-      g_key_file_set_boolean (service->settings, SETTINGS_GROUP_SERVICE,
-                              "UseMailboxInterface", service->use_mailbox_interface);
-      g_clear_error (&error);
-    }
-
-  service->mailbox_starttls = g_key_file_get_boolean (service->settings,
-                                                      SETTINGS_GROUP_SERVICE,
-                                                      "MailboxStartTLS", &error);
-  if (error)
-    {
-      service->mailbox_starttls = FALSE;
-
-      g_key_file_set_boolean (service->settings, SETTINGS_GROUP_SERVICE,
-                              "MailboxStartTLS", service->mailbox_starttls);
-      g_clear_error (&error);
-    }
-
   service->mailbox_legacytls = g_key_file_get_boolean (service->settings,
                                                        SETTINGS_GROUP_SERVICE,
                                                        "MailboxLegacyTLS", &error);
@@ -2238,6 +2122,10 @@ vvm_service_unref (struct vvm_service *service)
       g_free (service->mailbox_TUI_password_length);
       g_free (service->mailbox_vvm_type);
       g_free (service->mailbox_ca_bundle);
+      g_clear_pointer (&service->resolvers_ipv4_csv, g_free);
+      g_clear_pointer (&service->resolvers_ipv6_csv, g_free);
+      curl_slist_free_all (service->host_slist);
+      service->host_slist = NULL;
       service->settings = NULL;
     }
 
@@ -2279,6 +2167,8 @@ vvm_service_unregister (struct vvm_service *service)
       g_free (service->mailbox_TUI_password_length);
       g_free (service->mailbox_vvm_type);
       g_free (service->mailbox_ca_bundle);
+      g_clear_pointer (&service->resolvers_ipv4_csv, g_free);
+      g_clear_pointer (&service->resolvers_ipv6_csv, g_free);
       service->settings = NULL;
     }
 
@@ -2292,7 +2182,6 @@ vvm_service_unregister (struct vvm_service *service)
 
   g_clear_pointer (&service->country_code, g_free);
   g_clear_pointer (&service->path, g_free);
-  g_clear_pointer (&service->interface, g_free);
   g_clear_pointer (&service->own_number, g_free);
 
   if (service->mailbox_active)
