@@ -73,8 +73,8 @@ vvm_util_create_att_usa_status_sms (const char *carrier_prefix)
 }
 
 /*
- * 'STATE?' works for France Free Mobile, but not Vodafone au. However,
- * 'STATE' works for both. As of now, just changed all of them to 'STATE'
+ * 'STATE?' works for France Free Mobile However, 'STATE' reliably works
+ * for several carriers. As of now, just changed all of them to 'STATE'
  * and we can work out other issues if they come up later
  */
 static char
@@ -353,7 +353,7 @@ void
 vvm_util_parse_sync_ios_sms_message (const char                 *message,
                                      struct sms_control_message *sms_msg)
 {
-  DBG ("FreeMobile Sync message contain only the number of new messages.");
+  DBG ("iOS Sync message contain only the number of new messages.");
   sms_msg->sync_status_reason = SYNC_SMS_MAILBOX_UPDATE;
 }
 
@@ -520,7 +520,7 @@ decode (const char *input)
   decoded_string = g_strndup ((char *)base_64_decoded, base64_out_len);
   for (int i = 0; i < strlen (decoded_string); i++)
     {
-      char *decode[] = {
+      const char *decode[] = {
         "XTQ^ZSUU_Y00000",
         "YUP_[RTT^X11111",
         "ZVS\\XQWW][22222",
@@ -810,20 +810,21 @@ static char
 vvm_util_decode_vvm_single_email_attachment (const char *attachment,
                                              const char *folderpath)
 {
-  DBG ("Decoding Attachment");
   g_auto(GStrv) lines = NULL;
   g_autofree char *contentencoding = NULL;
   g_autofree char *contenttype = NULL;
   g_autofree char *duration = NULL;
   g_autofree char *attributes = NULL;
   g_autofree char *string_to_decode = NULL;
-  int attachment_line;
+  int attachment_line = 1;
   int found_attachment = FALSE;
   int base64_encoded;
 
   g_auto(GStrv) attribute_parse = NULL;
   g_autofree char *filename = NULL;
+  GString *decoded = g_string_new (NULL);
 
+  DBG ("Decoding Attachment");
   lines = g_strsplit_set (attachment, "\r\n", -1);
   //Decode Attachment headers first
   for (int i = 0; lines[i] != NULL; i++)
@@ -901,7 +902,6 @@ vvm_util_decode_vvm_single_email_attachment (const char *attachment,
   DBG ("Reconstructing the message!!");
   //AT&T USA linebreaks the base64 attachment every 60 characters
   //Why you ask? I have no idea.
-  GString *decoded = g_string_new (NULL);
   for (int i = attachment_line; lines[i] != NULL; i++)
     {
       if (lines[i + 1] != NULL)
