@@ -55,6 +55,8 @@ test_number_decode (gconstpointer data)
 
   //AT&T US
   const char carrer_vvm_test2[] = "94183567";
+  //AT&T US: This is where the texts come from
+  const char carrer_vvm_test6[] = "1000000000";
 
   //T-mobile US and Mint Mobile US
   const char carrer_vvm_test3[] = "127";
@@ -151,6 +153,11 @@ test_number_decode (gconstpointer data)
   g_assert (valid);
   valid = phone_utils_is_valid (carrer_vvm_test5, "US");
   g_assert (!valid);
+
+  valid = phone_utils_simple_is_valid (carrer_vvm_test6);
+  g_assert (valid);
+  valid = phone_utils_is_valid (carrer_vvm_test6, "US");
+  g_assert (!valid);
 }
 
 int
